@@ -1,6 +1,9 @@
 //! # Orthogonal Chebyshev Base
-use crate::enums::BaseKind;
-use crate::traits::{FunspaceElemental, FunspaceExtended, FunspaceSize};
+use crate::enums::{BaseKind, TransformKind};
+use crate::traits::{
+    BaseElements, BaseFromOrtho, BaseGradient, BaseMatOpDiffmat, BaseMatOpLaplacian,
+    BaseMatOpStencil, BaseSize, BaseTransform,
+};
 use crate::types::{FloatNum, ScalarNum};
 use ndarray::{s, Array2};
 use rustdct::{Dct1, DctPlanner};
@@ -79,7 +82,7 @@ impl<A: FloatNum> Chebyshev<A> {
     ///
     /// # Panics
     /// - Num conversion fails
-    /// - deriv > 2: Not implemented
+    /// - deriv > 4: Not implemented
     #[must_use]
     #[allow(clippy::cast_precision_loss)]
     pub fn _dmat(n: usize, deriv: usize) -> Array2<A> {
@@ -100,6 +103,31 @@ impl<A: FloatNum> Chebyshev<A> {
                     }
                 }
             }
+        } else if deriv == 3 {
+            for p in 0..n {
+                let p2 = p * p;
+                for q in p + 3..n {
+                    let q2 = q * q;
+                    if (p + q) % 2 != 0 {
+                        dmat[[p, q]] =
+                            (q * (q2 * (q2 - 2) - 2 * q2 * p2 + p2 * p2 - 2 * p2 + 1)) as f64 / 4.;
+                    }
+                }
+            }
+        } else if deriv == 4 {
+            for p in 0..n {
+                let (p2, p4) = (p * p, p * p * p * p);
+                for q in p + 4..n {
+                    let (q2, q4) = (q * q, q * q * q * q);
+                    if (p + q) % 2 == 0 {
+                        dmat[[p, q]] = (q
+                            * (q2 * (q2 - 4) * (q2 - 4) + 3 * q2 * p4 - p2 * p4 + 8 * p4
+                                - 16 * p2
+                                - 3 * q4 * p2)) as f64
+                            / 24.;
+                    }
+                }
+            }
         } else {
             todo!()
         }
@@ -123,15 +151,15 @@ impl<A: FloatNum> Chebyshev<A> {
     ///
     /// # Panics
     /// - Num conversion fails
-    /// - deriv > 2: Not implemented
+    /// - deriv > 4: Not implemented
     #[must_use]
     #[allow(clippy::cast_precision_loss)]
     pub fn _pinv(n: usize, deriv: usize) -> Array2<A> {
-        assert!(
-            !(deriv > 2),
-            "pinv does only support deriv's 1 & 2, got {}",
-            deriv
-        );
+        // assert!(
+        //     !(deriv > 2),
+        //     "pinv does only support deriv's 1 & 2, got {}",
+        //     deriv
+        // );
         let mut pinv = Array2::<f64>::zeros([n, n]);
         if deriv == 1 {
             pinv[[1, 0]] = 1.;
@@ -152,6 +180,57 @@ impl<A: FloatNum> Chebyshev<A> {
             for i in 2..n - 4 {
                 pinv[[i, i + 2]] = 1. / (4 * i * (i + 1)) as f64; // diag + 2
             }
+        } else if deriv == 3 {
+            // diag - 3
+            for i in 3..n {
+                let d = 8 * i * (i - 1) * (i - 2);
+                pinv[[i, i - 3]] = 1. / d as f64;
+            }
+            pinv[[3, 0]] *= 2.;
+            // diag - 1
+            for i in 3..n - 2 {
+                let d = 8 * (i + 1) * (i - 2) * i;
+                pinv[[i, i - 1]] = -3. / d as f64;
+            }
+            // diag + 1
+            for i in 3..n - 4 {
+                let d = 8 * (i - 1) * (i + 2) * i;
+                pinv[[i, i + 1]] = 3. / d as f64;
+            }
+            // diag + 3
+            for i in 3..n - 6 {
+                let d = 8 * i * (i + 1) * (i + 2);
+                pinv[[i, i + 3]] = -1. / d as f64;
+            }
+        } else if deriv == 4 {
+            // diag - 4
+            for i in 4..n {
+                let d = 16 * i * (i - 1) * (i - 2) * (i - 3);
+                pinv[[i, i - 4]] = 1. / d as f64;
+            }
+            pinv[[4, 0]] *= 2.;
+            // diag - 2
+            for i in 4..n - 2 {
+                let d = 4 * (i - 3) * (i - 1) * i * (i + 1);
+                pinv[[i, i - 2]] = -1. / d as f64;
+            }
+            // diag + 0
+            for i in 4..n - 4 {
+                let d = 8 * (i - 2) * (i - 1) * (i + 1) * (i + 2);
+                pinv[[i, i]] = 3. / d as f64;
+            }
+            // diag + 2
+            for i in 4..n - 6 {
+                let d = 4 * (i - 1) * i * (i + 1) * (i + 3);
+                pinv[[i, i + 2]] = -1. / d as f64;
+            }
+            // diag + 4
+            for i in 4..n - 8 {
+                let d = 16 * i * (i + 1) * (i + 2) * (i + 3);
+                pinv[[i, i + 4]] = 1. / d as f64;
+            }
+        } else {
+            todo!()
         }
         //pinv
         pinv.mapv(|elem| A::from_f64(elem).unwrap())
@@ -168,7 +247,7 @@ impl<A: FloatNum> Chebyshev<A> {
     }
 }
 
-impl<A: FloatNum> FunspaceSize for Chebyshev<A> {
+impl<A: FloatNum> BaseSize for Chebyshev<A> {
     /// Size in physical space
     #[must_use]
     fn len_phys(&self) -> usize {
@@ -188,52 +267,117 @@ impl<A: FloatNum> FunspaceSize for Chebyshev<A> {
     }
 }
 
-impl<A: FloatNum> FunspaceExtended for Chebyshev<A> {
-    type Real = A;
-
-    type Spectral = A;
+impl<A: FloatNum> BaseElements for Chebyshev<A> {
+    /// Real valued scalar type
+    type RealNum = A;
 
     /// Return kind of base
     fn base_kind(&self) -> BaseKind {
         BaseKind::Chebyshev
     }
 
+    /// Return kind of transform
+    fn transform_kind(&self) -> TransformKind {
+        TransformKind::R2r
+    }
+
     /// Coordinates in physical space
-    fn get_nodes(&self) -> Vec<A> {
+    fn coords(&self) -> Vec<A> {
         Chebyshev::nodes(self.len_phys())
     }
-
-    /// Mass matrix
-    fn mass(&self) -> Array2<A> {
-        Array2::<A>::eye(self.len_spec())
-    }
-
-    /// Explicit differential operator
-    fn diffmat(&self, deriv: usize) -> Array2<A> {
-        Self::_dmat(self.n, deriv)
-    }
-
-    /// Laplacian $ L $
-    fn laplace(&self) -> Array2<A> {
-        Self::_dmat(self.n, 2)
-    }
-
-    /// Pseudoinverse mtrix of Laplacian $ L^{-1} $
-    fn laplace_inv(&self) -> Array2<A> {
-        Self::_pinv(self.n, 2)
-    }
-
-    /// Pseudoidentity matrix of laplacian $ L^{-1} L $
-    fn laplace_inv_eye(&self) -> Array2<A> {
-        Self::_pinv_eye(self.n, 2)
-    }
 }
 
-impl<A: FloatNum> FunspaceElemental for Chebyshev<A> {
-    type Physical = A;
+impl<A: FloatNum> BaseMatOpDiffmat for Chebyshev<A> {
+    /// Scalar type of matrix
+    type NumType = A;
 
-    type Spectral = A;
+    /// Explicit differential operator $ D $
+    ///
+    /// Matrix-based version of [`BaseGradient::gradient()`]
+    fn diffmat(&self, deriv: usize) -> Array2<Self::NumType> {
+        assert!(deriv > 0);
+        Self::_dmat(self.n, deriv)
+    }
 
+    /// Explicit inverse of differential operator $ D^* $
+    ///
+    /// Returns ``(D_pinv, I_pinv)``, where `D_pinv` is the pseudoinverse
+    /// and ``I_pinv`` the corresponding pseudoidentity matrix, such
+    /// that
+    ///
+    /// ```text
+    /// D_pinv @ D = I_pinv
+    /// ```
+    ///
+    /// Can be used as a preconditioner.
+    fn diffmat_pinv(&self, deriv: usize) -> (Array2<Self::NumType>, Array2<Self::NumType>) {
+        assert!(deriv > 0);
+        (Self::_pinv(self.n, deriv), Self::_pinv_eye(self.n, deriv))
+    }
+}
+
+impl<A: FloatNum> BaseMatOpStencil for Chebyshev<A> {
+    /// Scalar type of matrix
+    type NumType = A;
+
+    /// Transformation stencil composite -> orthogonal space
+    fn stencil(&self) -> Array2<Self::NumType> {
+        Array2::<A>::eye(self.len_spec())
+    }
+
+    /// Inverse of transformation stencil
+    fn stencil_inv(&self) -> Array2<Self::NumType> {
+        Array2::<A>::eye(self.len_spec())
+    }
+}
+
+impl<A: FloatNum> BaseMatOpLaplacian for Chebyshev<A> {
+    /// Scalar type of matrix
+    type NumType = A;
+
+    /// Laplacian $ L $
+    fn laplacian(&self) -> Array2<Self::NumType> {
+        self.diffmat(2)
+    }
+
+    /// Pseudoinverse matrix of Laplacian $ L^{-1} $
+    ///
+    /// Returns pseudoinverse and pseudoidentity,i.e
+    /// ``(D_pinv, I_pinv)``
+    ///
+    /// ```text
+    /// D_pinv @ D = I_pinv
+    /// ``
+    fn laplacian_pinv(&self) -> (Array2<Self::NumType>, Array2<Self::NumType>) {
+        self.diffmat_pinv(2)
+    }
+}
+
+impl<A, T> BaseFromOrtho<T> for Chebyshev<A>
+where
+    A: FloatNum,
+    T: ScalarNum,
+{
+    /// Composite space coefficients -> Orthogonal space coefficients
+    fn to_ortho_slice(&self, indata: &[T], outdata: &mut [T]) {
+        for (y, x) in outdata.iter_mut().zip(indata.iter()) {
+            *y = *x;
+        }
+    }
+
+    /// Orthogonal space coefficients -> Composite space coefficients
+    fn from_ortho_slice(&self, indata: &[T], outdata: &mut [T]) {
+        for (y, x) in outdata.iter_mut().zip(indata.iter()) {
+            *y = *x;
+        }
+    }
+}
+
+impl<A, T> BaseGradient<T> for Chebyshev<A>
+where
+    A: FloatNum,
+    T: ScalarNum,
+{
     /// Differentiate Vector `n_times` using recurrence relation
     /// of chebyshev polynomials.
     ///
@@ -243,17 +387,17 @@ impl<A: FloatNum> FunspaceElemental for Chebyshev<A> {
     /// # Example
     /// Differentiate Chebyshev
     /// ```
-    /// use funspace::traits::FunspaceElemental;
+    /// use funspace::traits::BaseGradient;
     /// use funspace::chebyshev::Chebyshev;
     /// use funspace::utils::approx_eq;
     /// let mut ch = Chebyshev::<f64>::new(4);
     /// let indata: Vec<f64> = vec![1., 2., 3., 4.];
     /// let mut outdata: Vec<f64> = vec![0.; 4];
     /// // Differentiate twice
-    /// ch.differentiate_slice(&indata, &mut outdata, 2);
+    /// ch.gradient_slice(&indata, &mut outdata, 2);
     /// approx_eq(&outdata, &vec![12., 96.,  0.,  0.]);
     /// ```
-    fn differentiate_slice<T: ScalarNum>(&self, indata: &[T], outdata: &mut [T], n_times: usize) {
+    fn gradient_slice(&self, indata: &[T], outdata: &mut [T], n_times: usize) {
         assert!(outdata.len() == self.m);
         // Copy over
         for (y, x) in outdata.iter_mut().zip(indata.iter()) {
@@ -280,6 +424,12 @@ impl<A: FloatNum> FunspaceElemental for Chebyshev<A> {
             }
         }
     }
+}
+
+impl<A: FloatNum> BaseTransform for Chebyshev<A> {
+    type Physical = A;
+
+    type Spectral = A;
 
     fn forward_slice(&self, indata: &[Self::Physical], outdata: &mut [Self::Spectral]) {
         // Check input
@@ -324,22 +474,6 @@ impl<A: FloatNum> FunspaceElemental for Chebyshev<A> {
         // Transform via dct
         self.plan_dct.process_dct1(outdata);
     }
-
-    /// Composite space coefficients -> Orthogonal space coefficients
-    fn to_ortho_slice<T: Copy>(&self, indata: &[T], outdata: &mut [T]) {
-        // panic!("Function space Chebyshev is already orthogonal");
-        for (y, x) in outdata.iter_mut().zip(indata.iter()) {
-            *y = *x;
-        }
-    }
-
-    /// Orthogonal space coefficients -> Composite space coefficients
-    fn from_ortho_slice<T: Copy>(&self, indata: &[T], outdata: &mut [T]) {
-        // panic!("Function space Chebyshev is already orthogonal");
-        for (y, x) in outdata.iter_mut().zip(indata.iter()) {
-            *y = *x;
-        }
-    }
 }
 
 #[cfg(test)]
@@ -355,4 +489,21 @@ mod test {
         cheby.forward_slice(&indata, &mut outdata);
         approx_eq(&outdata, &vec![2.5, 1.33333333, 0., 0.16666667]);
     }
+
+    #[test]
+    fn test_chebyshev_pinv() {
+        let n = 27;
+        let ch = Chebyshev::<f64>::new(n);
+        // construct b4 by matmul
+        let (b2, _) = ch.diffmat_pinv(2);
+        let mut b4_v2 = b2.dot(&b2);
+        b4_v2.slice_mut(ndarray::s![..4, ..]).fill(0.);
+        b4_v2.slice_mut(ndarray::s![.., n - 4..]).fill(0.);
+        // get b4 directly
+        let (b4, _) = ch.diffmat_pinv(4);
+        // compare
+        for (x, y) in b4_v2.iter().zip(b4.iter()) {
+            assert!((x - y).abs() < 1e-6);
+        }
+    }
 }
