@@ -1,6 +1,9 @@
 use crate::chebyshev::{Chebyshev, ChebyshevComposite};
 use crate::fourier::{FourierC2c, FourierR2c};
-use crate::traits::{FunspaceElemental, FunspaceExtended, FunspaceSize};
+use crate::traits::{
+    BaseElements, BaseFromOrtho, BaseGradient, BaseMatOpDiffmat, BaseMatOpLaplacian,
+    BaseMatOpStencil, BaseSize, BaseTransform,
+};
 use crate::types::{FloatNum, ScalarNum};
 use ndarray::Array2;
 use num_complex::Complex;
@@ -35,6 +38,16 @@ impl_funspace_elemental_for_base!(BaseR2c, A, Complex<A>, FourierR2c);
 
 impl_funspace_elemental_for_base!(BaseC2c, Complex<A>, Complex<A>, FourierC2c);
 
+/// Set of transform kinds
+pub enum TransformKind {
+    /// real-to-real
+    R2r,
+    /// real-to-complex
+    R2c,
+    /// complex-to-complex
+    C2c,
+}
+
 /// All available function spaces
 #[derive(Debug, Clone, Copy)]
 pub enum BaseKind {
@@ -46,6 +59,10 @@ pub enum BaseKind {
     ChebNeumann,
     /// Chebyshev dirichlet - neumann base
     ChebDirichletNeumann,
+    /// Chebyshev biharmonic base A
+    ChebBiHarmonicA,
+    /// Chebyshev biharmonic base B
+    ChebBiHarmonicB,
     /// Fourier real to complex
     FourierR2c,
     /// Fourier complex to complex
@@ -59,6 +76,8 @@ impl std::fmt::Display for BaseKind {
             BaseKind::ChebDirichlet => write!(f, "ChebDirichlet"),
             BaseKind::ChebNeumann => write!(f, "ChebNeumann"),
             BaseKind::ChebDirichletNeumann => write!(f, "ChebDirichletNeumann"),
+            BaseKind::ChebBiHarmonicA => write!(f, "ChebBiHarmonicA"),
+            BaseKind::ChebBiHarmonicB => write!(f, "ChebBiHarmonicB"),
             BaseKind::FourierR2c => write!(f, "FourierR2c"),
             BaseKind::FourierC2c => write!(f, "FourierC2c"),
         }
